@@ -1,42 +1,29 @@
+import contextvars
 import datetime
 import json
-from threading import local
 from time import time
 
 from django.utils.encoding import force_str
 
 from debug_toolbar import settings as dt_settings
-from debug_toolbar.utils import get_stack, get_template_info, tidy_stacktrace
+from debug_toolbar.utils import get_stack_trace, get_template_info
 
 try:
     from psycopg2._json import Json as PostgresJson
+    from psycopg2.extensions import STATUS_IN_TRANSACTION
 except ImportError:
     PostgresJson = None
+    STATUS_IN_TRANSACTION = None
+
+# Prevents SQL queries from being sent to the DB. It's used
+# by the TemplatePanel to prevent the toolbar from issuing
+# additional queries.
+allow_sql = contextvars.ContextVar("debug-toolbar-allow-sql", default=True)
 
 
 class SQLQueryTriggered(Exception):
     """Thrown when template panel triggers a query"""
 
-    pass
-
-
-class ThreadLocalState(local):
-    def __init__(self):
-        self.enabled = True
-
-    @property
-    def Wrapper(self):
-        if self.enabled:
-            return NormalCursorWrapper
-        return ExceptionCursorWrapper
-
-    def recording(self, v):
-        self.enabled = v
-
-
-state = ThreadLocalState()
-recording = state.recording  # export function
-
 
 def wrap_cursor(connection, panel):
     if not hasattr(connection, "_djdt_cursor"):
@@ -50,14 +37,23 @@ def wrap_cursor(connection, panel):
             # See:
             # https://github.com/jazzband/django-debug-toolbar/pull/615
             # https://github.com/jazzband/django-debug-toolbar/pull/896
-            return state.Wrapper(
-                connection._djdt_cursor(*args, **kwargs), connection, panel
-            )
+            if allow_sql.get():
+                wrapper = NormalCursorWrapper
+            else:
+                wrapper = ExceptionCursorWrapper
+            return wrapper(connection._djdt_cursor(*args, **kwargs), connection, panel)
 
         def chunked_cursor(*args, **kwargs):
-            return state.Wrapper(
-                connection._djdt_chunked_cursor(*args, **kwargs), connection, panel
-            )
+            # prevent double wrapping
+            # solves https://github.com/jazzband/django-debug-toolbar/issues/1239
+            cursor = connection._djdt_chunked_cursor(*args, **kwargs)
+            if not isinstance(cursor, BaseCursorWrapper):
+                if allow_sql.get():
+                    wrapper = NormalCursorWrapper
+                else:
+                    wrapper = ExceptionCursorWrapper
+                return wrapper(cursor, connection, panel)
+            return cursor
 
         connection.cursor = cursor
         connection.chunked_cursor = chunked_cursor
@@ -66,12 +62,29 @@ def wrap_cursor(connection, panel):
 
 def unwrap_cursor(connection):
     if hasattr(connection, "_djdt_cursor"):
+        # Sometimes the cursor()/chunked_cursor() methods of the DatabaseWrapper
+        # instance are already monkey patched before wrap_cursor() is called.  (In
+        # particular, Django's SimpleTestCase monkey patches those methods for any
+        # disallowed databases to raise an exception if they are accessed.)  Thus only
+        # delete our monkey patch if the method we saved is the same as the class
+        # method.  Otherwise, restore the prior monkey patch from our saved method.
+        if connection._djdt_cursor == connection.__class__.cursor:
+            del connection.cursor
+        else:
+            connection.cursor = connection._djdt_cursor
         del connection._djdt_cursor
-        del connection.cursor
-        del connection.chunked_cursor
+        if connection._djdt_chunked_cursor == connection.__class__.chunked_cursor:
+            del connection.chunked_cursor
+        else:
+            connection.chunked_cursor = connection._djdt_chunked_cursor
+        del connection._djdt_chunked_cursor
 
 
-class ExceptionCursorWrapper:
+class BaseCursorWrapper:
+    pass
+
+
+class ExceptionCursorWrapper(BaseCursorWrapper):
     """
     Wraps a cursor and raises an exception on any operation.
     Used in Templates panel.
@@ -84,7 +97,7 @@ class ExceptionCursorWrapper:
         raise SQLQueryTriggered()
 
 
-class NormalCursorWrapper:
+class NormalCursorWrapper(BaseCursorWrapper):
     """
     Wraps a cursor and logs queries.
     """
@@ -128,16 +141,20 @@ class NormalCursorWrapper:
             return "(encoded string)"
 
     def _record(self, method, sql, params):
+        alias = self.db.alias
+        vendor = self.db.vendor
+
+        if vendor == "postgresql":
+            # The underlying DB connection (as opposed to Django's wrapper)
+            conn = self.db.connection
+            initial_conn_status = conn.status
+
         start_time = time()
         try:
             return method(sql, params)
         finally:
             stop_time = time()
             duration = (stop_time - start_time) * 1000
-            if dt_settings.get_config()["ENABLE_STACKTRACES"]:
-                stacktrace = tidy_stacktrace(reversed(get_stack()))
-            else:
-                stacktrace = []
             _params = ""
             try:
                 _params = json.dumps(self._decode(params))
@@ -145,10 +162,6 @@ class NormalCursorWrapper:
                 pass  # object not JSON serializable
             template_info = get_template_info()
 
-            alias = getattr(self.db, "alias", "default")
-            conn = self.db.connection
-            vendor = getattr(conn, "vendor", "unknown")
-
             # Sql might be an object (such as psycopg Composed).
             # For logging purposes, make sure it's str.
             sql = str(sql)
@@ -163,7 +176,7 @@ class NormalCursorWrapper:
                 "raw_sql": sql,
                 "params": _params,
                 "raw_params": params,
-                "stacktrace": stacktrace,
+                "stacktrace": get_stack_trace(skip=2),
                 "start_time": start_time,
                 "stop_time": stop_time,
                 "is_slow": duration > dt_settings.get_config()["SQL_WARNING_THRESHOLD"],
@@ -179,12 +192,31 @@ class NormalCursorWrapper:
                     iso_level = conn.isolation_level
                 except conn.InternalError:
                     iso_level = "unknown"
+                # PostgreSQL does not expose any sort of transaction ID, so it is
+                # necessary to generate synthetic transaction IDs here.  If the
+                # connection was not in a transaction when the query started, and was
+                # after the query finished, a new transaction definitely started, so get
+                # a new transaction ID from logger.new_transaction_id().  If the query
+                # was in a transaction both before and after executing, make the
+                # assumption that it is the same transaction and get the current
+                # transaction ID from logger.current_transaction_id().  There is an edge
+                # case where Django can start a transaction before the first query
+                # executes, so in that case logger.current_transaction_id() will
+                # generate a new transaction ID since one does not already exist.
+                final_conn_status = conn.status
+                if final_conn_status == STATUS_IN_TRANSACTION:
+                    if initial_conn_status == STATUS_IN_TRANSACTION:
+                        trans_id = self.logger.current_transaction_id(alias)
+                    else:
+                        trans_id = self.logger.new_transaction_id(alias)
+                else:
+                    trans_id = None
+
                 params.update(
                     {
-                        "trans_id": self.logger.get_transaction_id(alias),
+                        "trans_id": trans_id,
                         "trans_status": conn.get_transaction_status(),
                         "iso_level": iso_level,
-                        "encoding": conn.encoding,
                     }
                 )
 
