@@ -1,4 +1,3 @@
-from collections import OrderedDict
 from contextlib import contextmanager
 from os.path import normpath
 from pprint import pformat, saferepr
@@ -13,7 +12,7 @@ from django.urls import path
 from django.utils.translation import gettext_lazy as _
 
 from debug_toolbar.panels import Panel
-from debug_toolbar.panels.sql.tracking import SQLQueryTriggered, recording
+from debug_toolbar.panels.sql.tracking import SQLQueryTriggered, allow_sql
 from debug_toolbar.panels.templates import views
 
 # Monkey-patch to enable the template_rendered signal. The receiver returns
@@ -39,10 +38,10 @@ def _request_context_bind_template(self, template):
     self.template = template
     # Set context processors according to the template engine's settings.
     processors = template.engine.template_context_processors + self._processors
-    self.context_processors = OrderedDict()
+    self.context_processors = {}
     updates = {}
     for processor in processors:
-        name = "{}.{}".format(processor.__module__, processor.__name__)
+        name = f"{processor.__module__}.{processor.__name__}"
         context = processor(self.request)
         self.context_processors[name] = context
         updates.update(context)
@@ -118,7 +117,7 @@ class TemplatesPanel(Panel):
                                 value.model._meta.label,
                             )
                         else:
-                            recording(False)
+                            token = allow_sql.set(False)
                             try:
                                 saferepr(value)  # this MAY trigger a db query
                             except SQLQueryTriggered:
@@ -130,7 +129,7 @@ class TemplatesPanel(Panel):
                             else:
                                 temp_layer[key] = value
                             finally:
-                                recording(True)
+                                allow_sql.reset(token)
                     pformatted = pformat(temp_layer)
                     self.pformat_layers.append((context_layer, pformatted))
                 context_list.append(pformatted)
